from abc import ABC, abstractmethod
from datetime import datetime
from ali_autodrive.parallel_compute.biz_enum.TaskStatusEnum import TaskStatusEnum

'''
用户数据处理抽象类，用户需要实现该类的拆分和数据处理方法
'''

# 常量
# 健康状态为健康
HEALTH_TRUE = "true"
# 执行结果，成功
RESULT_TRUE = "true"
# 节点配置参数,SAAS层自定义KV
NODE_PARAMETER_MAP = "parameters"
# 父节点上下文列表
PARENT_CONTEXT_LIST = "parentContext"
# 数据分片
DATA_PARTITION_ID = "dataPartitionId"
# 数据分片列表
DATA_PARTITION_ID_LIST = "dataPartitionIds"
# 用户自定义上下文
USER_CONTEXT = "userContext"
# 拆分节点
PARTITION_JOB = "PARTITION_JOB"
# 节点配置参数 OSS路径
OSS_PREFIX = "ossPrefix"
# 工作目录
WORKSPACE = "workspace"


class DataProcessTaskAbstract(ABC):
    def __init__(self):
        # 任务ID
        self.task_id = None

        # 数据处理节点产出数据集ID
        self.data_partition_id = None

        # 数据拆分节点产出数据集列表
        self.data_partition_list = None

        # 当前处理数据分片对应的文件树
        self.data_partition_file_tree = None

        # 任务上下文
        self.context = {}

        # 用户上下文
        self.user_context = {}

        # 任务处理结果，新上下文
        self.new_context = {}

        # 是否健康，HEALTH_TRUE标示健康
        self.health = HEALTH_TRUE

        # 健康更新时间
        self.health_modify_time = datetime.now()

        # 运行状态
        self.status = TaskStatusEnum.RUNNING

        # 用户checkPoint
        self.check_point = None

        # 读写客户端
        self.read_client = None
        self.write_client = None

        # 初始化log服务
        self.logger = None

        # 初始分片为1 拆分节点每次拆分后加1
        self.data_partition_index = 1

        # 当前节点是否加载数据，加载则进行物理上传
        self.has_load_parent_data = False

        # 平台扩展能力
        self.handler = None

    def get_file_tree(self):
        return self.data_partition_file_tree

    def get_logger(self):
        return self.logger

    def get_workspace(self):
        return self.context[WORKSPACE]

    def save_data_partition(self, contents):
        self.write_client.save_data_partition(contents)

    # 合并节点，获取父节点列表
    def get_parent_data_partition_list(self):
        return self.context[PARENT_CONTEXT_LIST]

    # 获取父节点ID，非合并节点，只获取一个父节点信息
    def get_parent_data_partition_id(self):
        return self.context[PARENT_CONTEXT_LIST][0][DATA_PARTITION_ID]

    # 获取父节点用户上下文，非合并节点，只获取一个父节点信息
    def get_parent_data_partition_context(self):
        return self.context[PARENT_CONTEXT_LIST][0][USER_CONTEXT]

    # 获取节点配置参数
    def get_parameters(self):
        return self.context[NODE_PARAMETER_MAP]

    # 第一个几点，初始化父数据分片
    def init_parent_partition(self):
        # 如果数据分片不存在，尝试通过节点配置获取
        if PARENT_CONTEXT_LIST in self.context and self.context[PARENT_CONTEXT_LIST] is not None and len(
                self.context[PARENT_CONTEXT_LIST]) > 0:
            return

        data_partition_id = self.write_client.create_data_partition(self.context, True)
        parent_context_list = []
        parent_context = {DATA_PARTITION_ID: data_partition_id}
        parent_context_list.append(parent_context)
        self.context[PARENT_CONTEXT_LIST] = parent_context_list

    # 获取文件树，拆分节点需要物理拆分，需要手动load数据
    def scan_parent_file_tree(self):
        data_partition_ids = []
        for parent_context in self.context[PARENT_CONTEXT_LIST]:
            data_partition_ids.append(parent_context[DATA_PARTITION_ID])

        self.data_partition_file_tree = self.read_client.scan_file_tree_by_partition_ids(data_partition_ids)

    # 加载文件树
    def load_parent_file_tree(self):
        self.has_load_parent_data = True

        data_partition_ids = []
        for parent_context in self.context[PARENT_CONTEXT_LIST]:
            data_partition_ids.append(parent_context[DATA_PARTITION_ID])

        self.read_client.load_unstructured_data_partition_by_partition_ids(data_partition_ids)
        self.data_partition_file_tree = self.read_client.scan_file_tree_by_partition_ids(data_partition_ids)

    # 设置用户上下文
    def set_context(self, user_context):
        self.user_context = user_context

    # 获取用户上下文
    def get_context(self):
        return self.user_context


    # 注册读写客户端
    def register_rw_client(self, read_client, write_client):
        self.read_client = read_client
        self.write_client = write_client

    # 更新健康状态
    def modify_health(self, health):
        self.health = health
        self.health_modify_time = datetime.now()
        return RESULT_TRUE

    # 数据处理代理
    def data_process_proxy(self):
        try:
            # 加载待处理数据
            self.load_parent_file_tree()

            # 初始化分片,加载后再创建分片，影响分片存储目录
            self.data_partition_id = self.write_client.create_data_partition(self.context, False)

            # 用户数据处理
            self.data_process(self.user_context)

            # 提交描述文件,
            self.write_client.submit_data_partition(self.data_partition_id)

            # 数据处理后置扩展点
            self.handler.post_handler()
        except Exception as e:
            self.logger.info("Data process has exception.", e)

            # 更新任务状态
            self.status = TaskStatusEnum.FAILED
            return

        # 设置新的上下文信息
        self.new_context = {DATA_PARTITION_ID: self.data_partition_id, USER_CONTEXT: self.user_context}

        # 更新任务状态
        self.status = TaskStatusEnum.SUCCEED

    # 拆分逻辑代理
    def data_partition_proxy(self):
        try:
            # 加载待处理数据
            self.scan_parent_file_tree()

            # 拆分数据分片列表
            self.data_partition(self.user_context)
        except Exception as e:
            self.logger.info("Data partition has exception.", e)

            # 更新任务状态
            self.status = TaskStatusEnum.FAILED
            return

        # 保存拆分结果
        self.new_context = {DATA_PARTITION_ID_LIST: self.data_partition_list, USER_CONTEXT: self.user_context}

        # 更新任务状态
        self.status = TaskStatusEnum.SUCCEED


    @abstractmethod
    def data_process(self, context):
        pass

    @abstractmethod
    def data_partition(self, context):
        pass
