import json
import time
from datetime import datetime
from concurrent.futures import ThreadPoolExecutor
from ali_autodrive.parallel_compute.data_store.WriteClient import WriteClient
from ali_autodrive.parallel_compute.data_store.ReadClient import ReadClient
from ali_autodrive.parallel_compute.biz_enum.TaskStatusEnum import TaskStatusEnum
from ali_autodrive.parallel_compute.executor_agent.LogUtil import *
from ali_autodrive.parallel_compute.handler.DataTransformHandler import DataTransformHandler
from ali_autodrive.parallel_compute.handler.DefaultHandler import DefaultHandler

'''
并行计算管理器
'''
# 常量
# 任务状态
STATUS = "status"
# 最大并发数
MAX_WORKERS = 5
# 节点步骤类型
STEP_TYPE = "stepType"
# 健康状态
HEALTH = "health"
# 执行结果，成功
RESULT_TRUE = "true"
# 执行结果，失败
RESULT_FAIL = "false"
# 健康更新时间
HEALTH_MODIFY_TIME = "health_modify_time"
# 数据分片
DATA_PARTITION_ID = "dataPartitionId"
# 并行计算任务
PARALLEL = "PARALLEL"
# 运行实例
INSTANCE_CODE = "instanceCode"
# 用户自定义上下文
USER_CONTEXT = "userContext"
# 下划线分隔符
UNDER_LINE_SEPARATOR = "_"
# 节点类型
NODE_TYPE = "nodeType"
# 任务流实例
FLOW_INSTANCE_ID = "flowInstanceId"
# 工作目录
WORKSPACE = "workspace"


class ParallelComputeManager:
    def __init__(self, processor, logger):
        # 任务集合
        self.processor_task_map = {}

        # 线程池初始化，设置最大线程数为5
        self.executor = ThreadPoolExecutor(max_workers=MAX_WORKERS)

        # 用户数据处理器
        self.processor = processor

        # 初始化log服务
        self.logger = logger

        # 是否开启DEBUG模式
        self.debug_mode = False

    @staticmethod
    def __get_task_id(context):
        return context[INSTANCE_CODE]

    def __create_task(self, context):
        # 临时兼容任务实例ID不存在的情况，自动生成
        self.__init_flow_instance_id(context)

        # 创建处理任务，设置任务ID
        processor_task = self.processor()
        task_id = self.__get_task_id(context)
        processor_task.task_id = task_id

        # 设置读写客户端，每个任务独享
        read_client = ReadClient(processor_task)
        write_client = WriteClient(processor_task)
        processor_task.register_rw_client(read_client, write_client)

        # 初始化日志服务
        instance_code = context[INSTANCE_CODE]
        processor_task.logger = get_log(instance_code)

        # 避免文件混用，每个实例创建自己的目录
        context[WORKSPACE] = context[WORKSPACE] + "/" + str(time.time_ns())

        # 设置上下文
        processor_task.context = context
        if USER_CONTEXT in context:
            processor_task.user_context = context[USER_CONTEXT]
        else:
            processor_task.user_context = {}

        # 初始化任务流第一个节点
        self.logger.info("init_parent_partition" + json.dumps(context))
        processor_task.init_parent_partition()

        # 任务处理扩展能力
        processor_task.handler = self.__get_handler(context, processor_task)

        # 计算框架记录处理中的任务
        self.processor_task_map[task_id] = processor_task

        # 返回任务
        return processor_task

    def submit_task(self, context):
        processor_task = self.__create_task(context)
        try:
            # 根据节点类型确定入口
            step_type = context[STEP_TYPE]
            if step_type == PARALLEL:
                self.executor.submit(processor_task.data_process_proxy)
            else:
                self.executor.submit(processor_task.data_partition_proxy)
        except RuntimeError as err:
            self.logger.info("Data processor has exception.", err)
            processor_task.status = TaskStatusEnum.FAILED
        return {}

    def get_task(self, context):
        task_id = self.__get_task_id(context)
        if task_id in self.processor_task_map:
            return self.processor_task_map[task_id]
        else:
            return None

    def kill(self, context):
        task = self.get_task(context)
        if task is None:
            raise RuntimeError('Task not exists, context:', context)

        self.get_task(context).status = TaskStatusEnum.KILLED
        return {}

    def __get_handler(self, context, processor):
        if self.debug_mode is True:
            return DefaultHandler(processor)

        step_type = context[STEP_TYPE]
        if step_type is None:
            return DefaultHandler(processor)

        if step_type == "PARALLEL":
            return DataTransformHandler(processor)

        return DefaultHandler(processor)

    @staticmethod
    def __init_flow_instance_id(context):
        if FLOW_INSTANCE_ID in context:
            return

        now = datetime.now()
        context[FLOW_INSTANCE_ID] = now.strftime('%Y%m%d%H%M%S')
