# -*- coding: utf-8 -*-
import base64
import io
import time
from abc import ABC

import PIL
from ali_autodrive.parallel_compute.HandlerAbstract import HandlerAbstract
from ali_autodrive.parallel_compute.utils.http_util import *

from ali_autodrive.parallel_compute.biz_enum.DataTypeEnum import DataTypeEnum
from ali_autodrive.parallel_compute.executor_agent.LogUtil import *

from PIL import Image as pilImage

# 常量
# 节点配置参数
from ali_autodrive.parallel_compute.utils import pypcd

NODE_PARAMETERS = "parameters"
# 节点配置参数 OSS资源
OSS_SOURCE_CODE = "ossSourceCode"
# 节点配置参数 OSS路径
OSS_PREFIX = "ossPrefix"
# 查询数据集ID
QUERY_DATA_SET_ID = 'datasetId'
# 创建数据集ID
CREATE_DATA_SET_ID = 'dataSetId'
# 数据集名称
DATA_SET_NAME = 'dataSetName'
FLOW_NAME = "flowName"
FLOW_CODE = 'flowCode'
NODE_NAME = "nodeName"
NODE_CODE = "nodeCode"
# 任务流实例
FLOW_INSTANCE_ID = "flowInstanceId"
ROOT_PATH = "rootPath"
SCHEME_CODE = "schemeCode"
DATASET_ID = "datasetId"
# ES 索引
ES_INDEX_CAMERA = 'dwd_auto_dataset_item_camera_img_rt'
ES_INDEX_LIDAR = 'dwd_auto_dataset_item_lidar_rt'


class DataTransformHandler(HandlerAbstract, ABC):
    def __init__(self, processor):
        super(DataTransformHandler, self).__init__(processor)
        self.processor = processor

    def pre_handler(self):
        pass

    def post_handler(self):
        # 初始化参数
        context = self.processor.context
        data_set_id = context[FLOW_CODE] + "_" + context[NODE_CODE] + "_" + context[FLOW_INSTANCE_ID]

        # 查询数据集
        data_set = self.__get_data_set(data_set_id)

        # 创建数据集
        self.__create_data_set(context, data_set, data_set_id)

        print("dataset_id:", data_set_id)

        # 遍历文件，组装结构数据记录
        item_list = self.__convert_item_list(context, data_set_id)

        # 分批存储结构数据
        self.__save_item_list(item_list)

    @staticmethod
    def __get_data_set(data_set_id):
        # 校验数据集是否已经创建
        query_data_dictionary = {QUERY_DATA_SET_ID: data_set_id}
        return get_data_set(query_data_dictionary)

    def __create_data_set(self, context, data_set, data_set_id):
        data_set_name = context[FLOW_NAME] + "_" + context[NODE_NAME] + "_" + context[FLOW_INSTANCE_ID]
        try:
            if data_set is None:
                create_data_dictionary = {CREATE_DATA_SET_ID: data_set_id,
                                          DATA_SET_NAME: data_set_name,
                                          FLOW_CODE: context[FLOW_CODE],
                                          FLOW_INSTANCE_ID: context[FLOW_INSTANCE_ID],
                                          ROOT_PATH: context[NODE_PARAMETERS][OSS_PREFIX],
                                          SCHEME_CODE: context[NODE_PARAMETERS][OSS_SOURCE_CODE],
                                          }
                create_data_set(json.dumps(create_data_dictionary))
        except:
            data_set = self.__get_data_set(data_set_id)
            if data_set is None:
                raise RuntimeError("Create_data_set exception.")

    def __convert_item_list(self, context, data_set_id):
        # 获取描述文件及oss根路径
        local_desc_file = self.processor.read_client.get_partition_local_desc_file()

        item_list = {}
        with open(local_desc_file) as file:
            for line in file:
                item = json.loads(line)
                if "file_tag" in item:
                    if item["file_tag"]["header"]["data_type"] == DataTypeEnum.camera_img.name:
                        camera_item = self.__convert_camera_item(context, item, data_set_id)
                        if camera_item is not None:
                            if ES_INDEX_CAMERA not in item_list:
                                item_list[ES_INDEX_CAMERA] = []
                                item_list[ES_INDEX_CAMERA].append(json.dumps(camera_item))
                            else:
                                item_list[ES_INDEX_CAMERA].append(json.dumps(camera_item))

                    if item["file_tag"]["header"]["data_type"] == DataTypeEnum.lidar_pcd.name:
                        lidar_item = self.__convert_lidar_item(context, item, data_set_id)
                        if lidar_item is not None:
                            if ES_INDEX_LIDAR not in item_list:
                                item_list[ES_INDEX_LIDAR] = []
                                item_list[ES_INDEX_LIDAR].append(json.dumps(lidar_item))
                            else:
                                item_list[ES_INDEX_LIDAR].append(json.dumps(lidar_item))

        return item_list

    def __convert_camera_item(self, context, item, data_set_id):

        camera_item = {}
        file_tag = item["file_tag"]
        camera_item["dataset_id"] = data_set_id
        camera_item["version"] = 1
        # camera_item["tags"] =
        camera_item["sys_uuid"] = data_set_id + file_tag["header"]["vhc_id"] + '_' + file_tag["header"][
            "data_type"] + '_' + file_tag[
                                      "sensor_type_id"] + '_' + str(file_tag["header"]["timestamp_ns"])
        camera_item["vhc_id"] = file_tag["header"]["vhc_id"]
        camera_item["data_type"] = file_tag["header"]["data_type"]
        camera_item["sensor_type_id"] = file_tag["sensor_type_id"]
        camera_item["sensor_type"] = "camera"
        camera_item["start_time"] = time.strftime("%Y-%m-%d %H:%M:%S",
                                                  time.localtime(file_tag["header"]["timestamp_ns"] / 1000000000))
        camera_item["start_ts"] = file_tag["header"]["timestamp_ns"]
        camera_item["rec_time"] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        # camera_item["frame_id"] = file_tag["frame_id"]
        camera_item["oss_path"] = self.get_oss_path_by_relative_path(item["file_path"])
        camera_item["oss_schema_code"] = context[NODE_PARAMETERS][OSS_SOURCE_CODE]
        camera_item["seq"] = file_tag["header"]["timestamp_ns"]

        camera_item["height"] = file_tag["height"]
        camera_item["width"] = file_tag["width"]
        camera_item["format"] = file_tag["format"]

        # camera_item["encoding"] = file_tag["encoding"]
        # camera_item["step"] = file_tag["step"]

        local_path = self.get_local_path_by_relative_path(item["file_path"])
        camera_item["timg"] = self.__get_thumbnail(local_path, file_tag["format"])
        camera_item["doc_id"] = data_set_id + file_tag["header"]["vhc_id"] + '_' + file_tag["header"][
            "data_type"] + '_' + file_tag["sensor_type_id"] + '_' + str(file_tag["header"]["timestamp_ns"])

        return camera_item

    def __convert_lidar_item(self, context, item, data_set_id):

        lidar_item = {}
        file_tag = item["file_tag"]
        lidar_item["dataset_id"] = data_set_id
        lidar_item["version"] = 1
        # camera_item["tags"] =
        lidar_item["sys_uuid"] = data_set_id + file_tag["header"]["vhc_id"] + '_' + file_tag["header"]["data_type"] + '_' +\
                                 file_tag[ "sensor_type_id"] + '_' + str(file_tag["header"]["timestamp_ns"])
        lidar_item["vhc_id"] = file_tag["header"]["vhc_id"]
        lidar_item["data_type"] = file_tag["header"]["data_type"]
        lidar_item["sensor_type_id"] = file_tag["sensor_type_id"]
        lidar_item["sensor_type"] = "lidar"
        lidar_item["start_ts"] = file_tag["header"]["timestamp_ns"]
        lidar_item["start_time"] = time.strftime("%Y-%m-%d %H:%M:%S",
                                                 time.localtime(file_tag["header"]["timestamp_ns"] / 1000000000))
        lidar_item["rec_time"] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        # camera_item["frame_id"] = file_tag["frame_id"]
        lidar_item["oss_path"] = self.get_oss_path_by_relative_path(item["file_path"])
        lidar_item["comp_type"] = "drc"
        pcd_compression_oss_root_path = self.get_oss_path_by_relative_path(
            "/" + file_tag["sensor_type_id"] + "/drc" + item["file_path"])
        lidar_item["comp_oss_path"] = pcd_compression_oss_root_path

        lidar_item["oss_schema_code"] = context[NODE_PARAMETERS][OSS_SOURCE_CODE]
        lidar_item["seq"] = file_tag["header"]["timestamp_ns"]

        # camera_item["coord_type"] =
        # camera_item["dur_time_nsec"] =
        # camera_item["dur_step_time_nsec"] =

        # 点云压缩本地本和oss地址
        local_path = self.get_local_path_by_relative_path(item["file_path"])
        pc = pypcd.PointCloud.from_path(local_path)
        pc.save_pcd(local_path, compression='binary_compressed')

        self.processor.write_client.save_unstructured_data(local_path, pcd_compression_oss_root_path)
        return lidar_item

    def __get_thumbnail(self, image_path, format):
        ## 缩略图
        origin_im = pilImage.open(image_path)
        origin_im.thumbnail((256, 256), PIL.Image.ANTIALIAS)
        timg_buffer = io.BytesIO()

        # if format == "raw" or format == "RAW":
        #     origin_im.save(timg_buffer, format='RAW')
        #     timg = bytes("data:image/raw;base64,", encoding='utf-8') + base64.b64encode(timg_buffer.getvalue())
        if format == "png" or format == "PNG":
            origin_im.save(timg_buffer, format='PNG')
            timg = bytes("data:image/png;base64,", encoding='utf-8') + base64.b64encode(timg_buffer.getvalue())
        else:
            origin_im.save(timg_buffer, format='JPEG')
            timg = bytes("data:image/jpeg;base64,", encoding='utf-8') + base64.b64encode(timg_buffer.getvalue())
        # else:
        #     print("image type not support", format)

        return str(timg, 'utf-8')

    def __save_item_list(self, all_item_list):
        size = 5
        for index in all_item_list:
            item_list = []
            for item in all_item_list[index]:
                item_list.append(item)
                if len(item_list) >= size:
                    self.processor.write_client.save_structured_data(index, item_list)
                    item_list = []
            if len(item_list) > 0:
                self.processor.write_client.save_structured_data(index, item_list)

    def get_oss_path_by_relative_path(self, relative_path):
        if self.partition is None:
            self.partition = self.processor.read_client.get_data_partition()

        return self.partition["contentMap"]["OSS"]["rootPath"] + relative_path

    def get_local_path_by_relative_path(self, relative_path):
        if self.partition is None:
            self.partition = self.processor.read_client.get_data_partition()

        return self.processor.get_workspace() + relative_path
